package org.norgg.kittelutioning;
import nme.display.Sprite;
import nme.events.MouseEvent;
import nme.geom.Matrix;
import nme.text.TextField;
import org.norgg.kittelutioning.levels.Level;
import nme.display.GradientType;

/**
 * ...
 * @author Norgg
 */

class DNAEditor 
{
	static var instance: DNAEditor;
	
	var kitten: Kitten;
	
	var menuSprite: Sprite;
	var radiusSprite: Sprite;
	
	private function new() 
	{
		
	}
	
	private function refresh() {
		if (kitten == null) return;
		close();
		
		menuSprite = new Sprite();
		menuSprite.graphics.lineStyle(1, 0x000000, 1);
		menuSprite.graphics.beginFill(0xeeeeee, 0.8);
		menuSprite.graphics.drawRoundRect(0, 0, 50, 10 + height(), 3);
		menuSprite.addEventListener(MouseEvent.CLICK, click);
		Main.stage.addChild(menuSprite);
		
		radiusSprite = new Sprite();
		//radiusSprite.graphics.beginFill(0xff0000, 0.4);
		var m = new Matrix();
		m.createGradientBox(300, 300, 0, -150, -150);
		radiusSprite.graphics.beginGradientFill(GradientType.RADIAL, [0xffcccc, 0xff0000], [0.2, 0.6], [0, 255], m);
		radiusSprite.graphics.drawCircle(0, 0, 80);
		radiusSprite.x = kitten.x;
		radiusSprite.y = kitten.y;
		Main.stage.addChild(radiusSprite);
		
		var sprites = new List<Sprite>();
		
		var i = 0;
		for (gene in kitten.genes) {
			var sprite = new Sprite();
			sprites.add(sprite);

			var g = sprite.graphics;
			g.beginFill(0);
			g.drawRoundRect(10 + 10 * gene, 2.5, 5, 10, 1.5);
			
			var geneLabel = new TextField();
			geneLabel.selectable = false;
			geneLabel.defaultTextFormat = Main.greyFont;
			geneLabel.width = 30;
			geneLabel.height = 20;
			geneLabel.x = 60;
			geneLabel.y = 0;
			
			switch(i) {
				case 0:
					geneLabel.text = "->";
				case 1: 
					geneLabel.text = "<-";
				case 2:
					geneLabel.text = "^";
				case 3:
					geneLabel.text = "f";
				case 4:
					geneLabel.text = "I";
				case 5:
					geneLabel.text = "<->";
				case 6:
					geneLabel.text = "r";
				case 7:
					geneLabel.text = "g";
				case 8:
					geneLabel.text = "b";
				case 9:
					geneLabel.text = "O";
				case 10:
					geneLabel.text = "~";
				case 11:
					geneLabel.text = "/\\/\\";
				case 12:
					geneLabel.text = "()";
				
			}

			sprite.addChild(geneLabel);
			
			sprite.y = 5+15 * i;

			
			menuSprite.addChild(sprite);
			
			var geneIndex = i;
			var clickFunction = function(e) {
				kitten.genes[geneIndex] = (kitten.genes[geneIndex] + 1) % 4;
				e.stopImmediatePropagation();
				kitten.rebuild();
				refresh();
			};
			sprite.addEventListener(MouseEvent.CLICK, clickFunction);
			
			i++;
		}
		update();
	}
	
	function click(e: MouseEvent) {
		e.stopImmediatePropagation();
	}
	
	function height() {
		return kitten.genes.length * 15;
	}
	
	public function close() {
		if (menuSprite != null) {
			Main.stage.removeChild(menuSprite);
			Main.stage.removeChild(radiusSprite);
			menuSprite = null;
		}
	}
	
	public static function edit(kitten: Kitten) {
		if (instance == null) instance = new DNAEditor();
		instance.kitten = kitten;
		instance.refresh();
	}
	
	public function update() {
		if (menuSprite != null) {
			menuSprite.x = kitten.x + 82;
			menuSprite.y = kitten.y - kitten.height - height()/2 - 14;
			radiusSprite.x = kitten.x;
			radiusSprite.y = kitten.y;
		}
		if (!kitten.active) close();
	}
	
	public static function updateInstance() {
		if (instance != null && instance.kitten != null) {
			instance.update();
		}
	}
	
	public static function refreshInstance() {
		if (instance != null && instance.kitten != null) {
			instance.refresh();
		}
	}
	
	public static function closeInstance() {
		if (instance != null) instance.close();
	}
}