package org.norgg.kittelutioning;

import nape.geom.Vec2;
import nme.display.Graphics;
import nme.display.Sprite;
import nape.phys.Body;
import nape.phys.BodyType;
import nape.shape.Shape;
import nape.shape.Circle;
import nape.shape.Polygon;
import org.norgg.kittelutioning.levels.Level;

/**
 * ...
 * @author Norgg
 */

class Entity {
	public var x: Float;
	public var y: Float;
	var rotation: Float;
	var bodyType: BodyType;
	
	var sprite: Sprite;
	public var body: Body;
	var g: Graphics;
	var shapeArray: Array<Shape>;
	var drawColor = 0;
	
	public var active = false;
	
	public function new(pos: Vec2, rotation: Float, bodyType: BodyType) {
		x = pos.x;
		y = pos.y;
		this.rotation = rotation;
		this.bodyType = bodyType;
	}
	
	function shapes() {
		return new Array<Shape>();
	}
	
	public function color() { return 0; }
	
	public function drawShapes() {
		for (shape in shapeArray) {
			g.beginFill(drawColor);
			if (shape.isCircle()) {
				g.drawCircle(shape.castCircle.localCOM.x, shape.castCircle.localCOM.y, shape.castCircle.radius);
			} else if (shape.isPolygon()) {
				var first = true;
				for (point in shape.castPolygon.localVerts) {
					if (first) {
						g.moveTo(point.x, point.y);
						first = false;
					} else {
						g.lineTo(point.x, point.y);
					}
				}
			}
		}
	}
	
	public function init() {
		if (!active) {
			sprite = new Sprite();
			g = sprite.graphics;
			body = new Body(bodyType);
			body.position = new Vec2(x, y);
			body.rotation = rotation;
			body.userData.obj = this;
			
			shapeArray = shapes();
			
			for (shape in shapeArray) {
				body.shapes.add(shape);
			}
			
			drawColor = color();

			drawShapes();
			
			body.space = Main.space;
			Main.stage.addChildAt(sprite, 0);
			active = true;
		}
	}
	
	public function destroy() {
		if (active) {
			body.space = null;
			Main.stage.removeChild(sprite);
			active = false;
		}
	}
	
	public function rebuild() {
		var vel = body.velocity.copy();
		destroy();
		init();
		body.velocity = vel;
		update();
	}
	
	public function update() {
		sprite.x = x = body.position.x;
		sprite.y = y = body.position.y;
		rotation = body.rotation;
		sprite.rotation = 180/Math.PI * body.rotation;
	}
}