package org.norgg.kittelutioning;

import nape.constraint.AngleJoint;
import nape.constraint.MotorJoint;
import nape.constraint.PivotJoint;
import nape.constraint.WeldJoint;
import nape.dynamics.InteractionGroup;
import nape.geom.Vec2;
import nape.phys.BodyType;
import nape.phys.Material;
import nape.shape.Polygon;
import nape.shape.Shape;
import nme.events.MouseEvent;
import nme.media.SoundChannel;
import nme.media.Sound;
import nme.events.Event;
import nme.Assets;
import nape.callbacks.InteractionType;
import nape.dynamics.InteractionFilter;
import nape.constraint.Constraint;
import nape.phys.Compound;
import org.norgg.kittelutioning.levels.Exit;
import org.norgg.kittelutioning.levels.Level;

using Std;
using Math;

/**
 * ...
 * @author Norgg
 */

 //Yay, submitted to Ludum Dare 24! - 26-08-2012
 
class Kitten extends Entity
{
	public var genes: Array<Int>;
	static var baseSize = 20;
	
	public var height: Float = 20;
	public var width: Float = 20;
	
	public var onGround = false;
	
	static var jumpTime = 50;
	static var jumpTimer = 0;
	public static var numGenes = 13;
	
	public var material: Material;
	
	var filter: InteractionFilter;
	
	var legs: Array<Leg>;
	public var compound: Compound;
	public var head: Head;
	
	var currentSound: SoundChannel;
	
	static var sounds = { new Hash<Sound>(); }
	static var mews = ["mew1.wav", "mew2.wav"];
	
	public function new(x, y, filter) {
		this.filter = filter;
		super(new Vec2(x, y), 0, BodyType.DYNAMIC);
		genes = new Array<Int>();
		for (i in 0...numGenes) {
			genes[i] = 0;
		}
	}
	
	override function init() {
		height = baseSize * (1 + genes[4]/4) * 0.6;
		width  = baseSize * (1 + genes[5]/4);
		material = new Material(0.1, 0.1, 0.1, 1+genes[9], 0.01);

		super.init();
		
		
		body.setShapeFilters(filter);
		sprite.addEventListener(MouseEvent.MOUSE_DOWN, mouseDown);
		
		compound = new Compound();
		compound.space = Main.space;
		compound.bodies.add(body);
		
		var group = new InteractionGroup(true);
		body.group = group;
		
		legs = new Array<Leg>();
		
		for (i in 0...(4 - genes[11])) {
			var legX = 0.0;
			if (i == 0 || i == 1) {
				legX = x + width / 2;
			} else if (i == 2 || i == 3) {
				legX = x - width / 2 + 2;
			} else {
				legX = x;
			}
			
			var leg = new Leg(new Vec2(legX, y + height), this);
			leg.init();
			leg.body.group = group;
			compound.bodies.add(leg.body);
			
			var pivot = new PivotJoint(body, leg.body, new Vec2(legX - x, height / 2), new Vec2(leg.w / 2, -leg.h / 2));
			compound.constraints.add(pivot);
			
			var angle = new AngleJoint(body, leg.body, -0.5, 0.5);
			compound.constraints.add(angle);
			
			var motor = new MotorJoint(body, leg.body, 1, 1);
			compound.constraints.add(motor);
			
			leg.motor = motor;
			legs.push(leg);
			leg.body.cbTypes.add(Main.collide);
		}
		
		var headRadius = 6;
		
		head = new Head(body.position, headRadius, this);
		head.init();
		head.body.group = group;
		compound.bodies.add(head.body);
		
		var neckWeld = new WeldJoint(body, head.body, new Vec2(width / 2 + 3, -height - 1), new Vec2(0, -headRadius));
		compound.constraints.add(neckWeld);
		
		body.cbTypes.add(Main.collide);
	}
	
	override function shapes() {
		var shape: Shape = new Polygon(Polygon.box(width, height), material);

		return [shape];
	}
	
	override function color() {
		var r = (32 + genes[6] * 48);
		var g = (32 + genes[7] * 48);
		var b = (32 + genes[8] * 48);
		return (r<<16) + (g<<8) + (b);
	}
	
	override public function drawShapes() {
		super.drawShapes();
	}
	
	override function update() {
		head.update();
		for (leg in legs) leg.update();
		super.update();
		if (jumpTimer > 0) jumpTimer--;

		for (thisBody in compound.bodies) {
			for (interaction in thisBody.interactingBodies(InteractionType.SENSOR, 1)) {
				for (body in interaction.connectedBodies()) {
					if (body.userData.obj.is(Exit)) {
						Level.current.savedKittens.add(this);
						Level.current.aliveKittens.remove(this);
						destroy();
					}
				}
			}
		}
		
		//Angle for cat to be upright.
		var r = body.rotation - Math.PI / 2 + 0.2;
		if (r > 0) {
			body.torque = -(3 - genes[12]) * (r * 2).sin() * 400;
		} else {
			body.torque =  (3 - genes[12]) * (r * 2).sin() * 400;
		}
		
		var i = 0;
		for (leg in legs) {
			if (body.velocity.x.abs() > 5) {
				var rate = 15;
				if (Main.tick % 20 < 10) {
					rate *= -1;
				}
				if (i%2==0) {
					rate *= -1;
				}
				leg.motor.maxForce = 1000;
				leg.motor.rate = rate;
				leg.motor.active = true;
			} else {
				leg.motor.active = false;
			}
			i++;
		}
			
		if (onGround) {
			body.velocity.x = 0;
			body.velocity.x += genes[0] * 40;
			body.velocity.x -= genes[1] * 18;
			

			if (genes[2] > 0 && jumpTimer <= 0) {
				for (b in compound.bodies) b.velocity.y = -genes[2] * 150;
				jumpTimer = jumpTime * (4-genes[3]);
			}
		}
		
		if (genes[10] > 0) {
			body.applyLocalForce(new Vec2(0, -60 * genes[10]));
		}
		
		onGround = false;
	}
	
	public function mouseDown(e: MouseEvent) {
		if (!active) return;
		DNAEditor.edit(this);
		playSound("mew-query.wav");
		Main.mutateKitten = this;
		Main.mutateStartX = e.stageX;
		Main.mutateStartY = e.stageY;
		e.stopImmediatePropagation();
	}
	
	public function playSound(soundStr) {
		if (currentSound == null) {
			var sound: Sound;
			if (sounds.exists(soundStr)) {
				sound = sounds.get(soundStr);
			} else {
				sound = Assets.getSound("snd/" + soundStr);
				#if !debug
					if (sound == null) return; //"silently" fail.
				#end
				sounds.set(soundStr, sound);
			}
			currentSound = sound.play();
			currentSound.addEventListener(Event.SOUND_COMPLETE, function(e) { this.currentSound = null; } );
		}
	}
	
	public function interruptSound(soundStr) {
		if (currentSound != null) {
			currentSound.stop();
			currentSound = null;
		}
		playSound(soundStr);
	}
	
	public function randomMew() {
		playSound(mews[(Math.random() * mews.length).int()]);
	}
	
		
	public function breedWith(other) {
		var litterSize = 3 + (Math.random() * 2).int();
		
		var litter = new List<Kitten>();
		
		for (i in 0...litterSize) {
			var newKitten = Level.current.createKitten();
			
			for (gene in 0...genes.length) {
				
				var newGene = (genes[gene] + (other.genes[gene]) / 2).int();
				if (Math.random() < 0.1) {
					newGene = (newGene + 1) % 4;
				}
				if (newGene > 3) newGene = 3;
				if (newGene < 0) newGene = 0;
				newKitten.genes[gene] = newGene;
				
			}
			litter.add(newKitten);
		}
		
		return litter;
	}
	
	public function kill() {
		if (!active) return;
		Level.current.deadKittens.add(this);
		Level.current.aliveKittens.remove(this);
		destroy();
		for (i in 0...40) {
			var blood = new Blood(new Vec2(x+Math.random()*width, y+Math.random()*height));
			blood.init();
		}
		head = new Head(new Vec2(head.x, head.y), head.r, this);
		head.init();
		interruptSound("pop.wav");
	}
	
	override function destroy() {
		if (!active) return;
		compound.space = null;
		compound.bodies.clear();
		compound.constraints.clear();
		for (leg in legs) leg.destroy();
		head.destroy();
		super.destroy();
	}
}