package org.norgg.kittelutioning;

import haxe.Int64;
import nape.callbacks.CbType;
import nape.callbacks.InteractionCallback;
import nape.callbacks.InteractionListener;
import nape.geom.Vec2;
import nape.shape.Shape;
import nme.display.Sprite;
import nme.display.Stage;
import nme.display.StageScaleMode;
import nme.display.StageAlign;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.Lib;
import haxe.Timer;
import nape.space.Space;
import nape.callbacks.InteractionType;
import nape.callbacks.CbEvent;
import nme.text.TextFormat;
import nme.Assets;
import org.norgg.kittelutioning.levels.Level;

using Std;

/**
 * ...
 * @author Norgg
 */

class Main
{
	public static var stage: Stage;
	public static var space: Space;
	
	public static var gameTimer: Timer;
	
	public static var fps = 50;
	
	public static var w = 800;
	public static var h = 600;
	
	public static var font = { new TextFormat("_sans", 12, 0x000000); };
	public static var bigFont = { new TextFormat("_sans", 24, 0x000000); };
	public static var whiteFont = { new TextFormat("_sans", 12, 0xffffff); };
	public static var bigWhiteFont = { new TextFormat("_sans", 24, 0xffffff); };
	public static var redFont = { new TextFormat("_sans", 24, 0xff0000); };
	public static var greyFont = { new TextFormat("_sans", 12, 0x666666); };
	
	public static var mutateKitten: Kitten;
	public static var mutateStartX: Float = 0;
	public static var mutateStartY: Float = 0;
	
	public static var collide = new CbType();
	
	public static var tick = 0;
	
	static var geneIndices: Array<Int>;
	
	private static function init() 
	{
		tick = 0;
		
		stage = Lib.current.stage;
		stage.scaleMode = StageScaleMode.SHOW_ALL;
		stage.align = StageAlign.TOP;
		
		stage.addEventListener(MouseEvent.CLICK, click);
		stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMove);
		stage.addEventListener(MouseEvent.MOUSE_UP, mouseUp);

		space = new Space(new Vec2(0, 500));

		Level.next();
		
		geneIndices = new Array<Int>();
		for (i in 0...Kitten.numGenes) {
			geneIndices[i] = i;
		}
		
		//Shuffle gene indexes.
		for (i in 0...geneIndices.length) {
			var swp = Std.int(geneIndices.length * Math.random());
			var tmp = geneIndices[swp];
			geneIndices[swp] = geneIndices[i];
			geneIndices[i] = tmp;
		}


		if (gameTimer != null) {
			gameTimer.stop();
		}
		gameTimer = new Timer(Std.int(1000 / fps));
		gameTimer.run = update;
		update();
		space.listeners.add(new InteractionListener(CbEvent.ONGOING, InteractionType.COLLISION, collide, collide, onInteract));
		space.listeners.add(new InteractionListener(CbEvent.BEGIN, InteractionType.COLLISION, collide, collide, onInteract));
	}
	
	public static function update() {
		Level.current.update();
		Blood.updateAll();
		DNAEditor.updateInstance();
		space.step(1 / fps, 2, 2);
		tick++;
	}
	
	public static function click(e) {
		DNAEditor.closeInstance();
	}
	
	public static function onInteract(cb: InteractionCallback) {
		for (arbiter in cb.arbiters) {
			var s1 = arbiter.shape1;
			var s2 = arbiter.shape2;
			
			checkGround(s1, s2);
			checkGround(s2, s1);			
		}
	}
	
	public static function checkGround(s1: Shape, s2: Shape) {
		if (s1.body.userData.obj.is(Kitten)) {
			if (s2.body.userData.obj.is(Level)) {
				if (s2.userData.isGround) {
					s1.body.userData.obj.onGround = true;
				}
			}
		} else if (s1.body.userData.obj.is(Leg)) {
			if (s2.body.userData.obj.is(Level)) {
				if (s2.userData.isGround) {
					s1.body.userData.obj.kitten.onGround = true;
				}
			}
		}
	}
	
	
	public static function mouseMove(e: MouseEvent) {
		if (mutateKitten != null) {
			var dragged = new Vec2(
				e.stageX - mutateKitten.x,
				e.stageY - mutateKitten.y
			);
			var mutation = Std.int((mutateKitten.genes.length * ((Math.PI+dragged.angle) / (Math.PI * 2))));
			var size = Std.int(dragged.length / 20);
			
			if (size > 2.5) {
				mutateKitten.playSound("distress.wav");
			}
			
			if (size > 3) {
				mutateKitten.kill();
				mutateKitten = null;
			}
			else {
				mutateKitten.genes[geneIndices[mutation]] = size;
				DNAEditor.refreshInstance();
			}
		}
	}
	public static function mouseUp(e: MouseEvent) {
		if (mutateKitten != null && mutateKitten.active) {
			mutateKitten.rebuild();
		}
		mutateKitten = null;
		DNAEditor.closeInstance();
	}
		
	static public function main() 
	{
		init();
	}
	
	public function new() {
		init();
	}
}
