package org.norgg.kittelutioning.levels;

import nape.dynamics.InteractionGroup;
import nape.geom.Vec2;
import nape.phys.Body;
import nape.phys.BodyType;
import nape.shape.Shape;
import nape.shape.Polygon;
import nape.shape.Circle;
import nme.display.Graphics;
import nme.display.Sprite;
import nme.text.TextField;
import nme.text.TextFormat;
import nme.events.MouseEvent;
import org.norgg.kittelutioning.Kitten;
import org.norgg.kittelutioning.Main;
import nape.dynamics.InteractionFilter;
import nme.text.TextFieldAutoSize;

using Std;

/**
 * ...
 * @author Norgg
 */

class Level extends Entity
{	
	static var levels: Array<Level>;
	
	public var aliveKittens: List<Kitten>;
	public var deadKittens:  List<Kitten>;
	public var savedKittens: List<Kitten>;
	public var spawnKittens: List<Kitten>; //Kittens yet to be spawned.

	var entrancePos: Vec2;
	var entranceSprite: Sprite;
	
	var exitPos: Vec2;
	var exitSprite: Sprite;
	var exitBody: Body;
	
	static var levelGroup = 1;
	static var exitGroup = 2;
	static var kittenGroup = 4;
	
	var exitFilter: InteractionFilter;
	var kittenFilter: InteractionFilter;
	
	public static var current: Level;
	public static var levelNum = 0;
	
	var timeLeft = 0;
	
	static var spawnTime = 100;
	var spawnTimer = 0;
	
	var endTimer = 100;
	
	var kittensLeftLabel: TextField;
	var kittensSavedLabel: TextField;
	var timeLeftLabel: TextField;
	var levelLabel: TextField;
	
	var exitEnt: Exit;
	
	public function new() {
		super(new Vec2(0, 0), 0, BodyType.STATIC);
		entrancePos = entrance();

		exitFilter   = new InteractionFilter(exitGroup,   0,    exitGroup,   kittenGroup); //Exit doesn't interact with anything.
		kittenFilter = new InteractionFilter(kittenGroup, 0xff, kittenGroup, exitGroup);   //Kittens sense exit.
	}
	
	override function init() {
		super.init();
		aliveKittens = new List<Kitten>();
		deadKittens =  new List<Kitten>();
		savedKittens = new List<Kitten>();
		spawnKittens = new List<Kitten>();
		
		for (i in 0...initialKittens()) {
			spawnKittens.add(createKitten());
		}
		
		exitEnt = new Exit(exit(), exitFilter);
		exitEnt.init();
		
		var entranceSprite = new Sprite();
		entranceSprite.graphics.beginFill(0xff0000);
		entranceSprite.graphics.drawRect(entrancePos.x - 25, entrancePos.y - 15, 50, 30);
		entranceSprite.addEventListener(MouseEvent.CLICK, function(e) { spawnKitten(); } );
		sprite.addChild(entranceSprite);
		
		kittensLeftLabel = new TextField();
		kittensLeftLabel.defaultTextFormat = Main.bigFont;
		kittensLeftLabel.x = entrancePos.x - 8;
		kittensLeftLabel.y = entrancePos.y - 15;
		kittensLeftLabel.height = 30;
		kittensLeftLabel.width = 30;
		kittensLeftLabel.selectable = false;
		kittensLeftLabel.addEventListener(MouseEvent.CLICK, function(e) { spawnKitten(); } );
		sprite.addChild(kittensLeftLabel);
		
		kittensSavedLabel = new TextField();
		kittensSavedLabel.defaultTextFormat = Main.bigFont;
		kittensSavedLabel.x = exitEnt.x-9;
		kittensSavedLabel.y = exitEnt.y-8;
		kittensSavedLabel.height = 30;
		kittensSavedLabel.width = 20;
		kittensSavedLabel.selectable = false;
		sprite.addChild(kittensSavedLabel);
		
		levelLabel = new TextField();
		levelLabel.defaultTextFormat = Main.bigWhiteFont;
		levelLabel.x = 10;
		levelLabel.y = 10;
		levelLabel.height = 40;
		levelLabel.width  = 350;
		levelLabel.selectable = false;
		levelLabel.text = name();
		sprite.addChild(levelLabel);
		
		timeLeftLabel = new TextField();
		timeLeftLabel.defaultTextFormat = Main.bigWhiteFont;
		timeLeftLabel.x = 380;
		timeLeftLabel.y = 10;
		timeLeftLabel.height = 40;
		timeLeftLabel.selectable = false;
		sprite.addChild(timeLeftLabel);
		
		timeLeft = time();
		
		body.cbTypes.add(Main.collide);
		
		endTimer = 100;
	}
	
	public static function initLevels() {
		levels = [new Level1(), new Level2(), new Level3(), new Level4(), new Win()];
	}
	
	function entrance() { return new Vec2(100, 200); }
	function exit()     { return new Vec2(100, 400); }
	function name() { return "Level" + Level.levelNum; }
	
	override function update() {
		spawnTimer--;
		timeLeft--;
		if (spawnTimer <= 0) {
			spawnKitten();
		}
		
		for (kitten in aliveKittens) kitten.update();
		for (kitten in deadKittens) kitten.update();
		
		kittensSavedLabel.text = ""+savedKittens.length;
		kittensLeftLabel.text = "" + spawnKittens.length;
		
		if (timeLeft < 300) {
			timeLeftLabel.defaultTextFormat = Main.redFont;
			timeLeftLabel.x = 100;
			timeLeftLabel.y = 100;
			timeLeftLabel.width = 600;
			timeLeftLabel.text = "TERMINATING UNDERPERFORMING SUBJECTS IN " + (timeLeft / 50).int();
		} else {
			timeLeftLabel.text = ""+(timeLeft / 50).int();
		}
		
		if (timeLeft <= 0) {
			for (kitten in aliveKittens) kitten.kill();
		}
		
		if (spawnKittens.length + aliveKittens.length + savedKittens.length < 2) {
			endTimer--;
			if (endTimer <= 0) {
				Level.set(new Loss("Extinction. Lone survivor cannot breed."));
			}
		}
		if (spawnKittens.isEmpty() && aliveKittens.isEmpty() || timeLeft <= 0) {
			endTimer--;
			if (savedKittens.length >= 2) {
				timeLeftLabel.x = 200;
				timeLeftLabel.y = 250;
				timeLeftLabel.width = 400;
				timeLeftLabel.defaultTextFormat = Main.bigFont;
				timeLeftLabel.text = "BREEDING GENERATION " + (levelNum + 1);
			}

			if (endTimer <= 0) {
				if (savedKittens.length >= 2) {
					next();
				} else {
					var message = "";
					if (savedKittens.length == 1) {
						message = "Extinction. Lone survivor cannot breed.";
					} else {
						message = "Extinction. You killed them all.";
					}
					Level.set(new Loss(message));
				}
			}
		}
	}
	
	function mutagens()       { return 30; }
	function initialKittens() { return 5; }
	function time()           { return 3000; }
	
	public function createKitten() {
		return new Kitten(entrancePos.x, entrancePos.y, kittenFilter);
	}
	
	function spawnKitten() {
		spawnTimer = spawnTime;
		if (spawnKittens.length > 0) {
			var kitten = spawnKittens.pop();
			kitten.init();
			kitten.randomMew();
			aliveKittens.add(kitten);
		}
	}
	
	override function destroy() {
		super.destroy();
		if (exitEnt != null) {
			exitEnt.destroy();
		}
		
		if (aliveKittens != null) {
			for (kitten in aliveKittens) kitten.destroy();
		}
		
		//Clean up decapitated heads.
		if (deadKittens != null) {
			for (kitten in deadKittens) {
				if (kitten.head != null) {
					kitten.head.destroy();
				}
			}
		}
		
		//Clean up blood.
		Blood.cleanAll();
	}
	
	public static function start(levelNum) {
		if (levels == null) initLevels();

		
		var newLevel = levels[0];
		if (levelNum > 0 && levelNum <= levels.length) {
			newLevel = levels[levelNum - 1];
		}
		set(newLevel);
	}
	
	public static function set(level: Level) {
		if (current != null) {
			current.destroy();
		}
		
		current = level;
		current.init();
	}
	
	public static function next() {
		levelNum++;
		if (current != null) {
			var kittensCarried = current.savedKittens;
			start(levelNum);
			
			if (kittensCarried != null) {
				current.spawnKittens.clear();
				for (i in 0...(kittensCarried.length / 2).int()) {
					var k1: Kitten = kittensCarried.pop();
					var k2: Kitten = kittensCarried.pop();
					var newKittens = k1.breedWith(k2);
					for (kitten in newKittens) {
						//Cap at 10 cats.
						if (current.spawnKittens.length < 10) {
							current.spawnKittens.add(kitten);
						}
					}
				}
			}
		} else {
			start(levelNum);
		}
	}
	
	public function wall(x: Float, y: Float, w: Float, h: Float) {
		var s = 50;
		return new Polygon(Polygon.rect(s*x, s*y, s*w, s*h)).castShape;
	}
	
	public function wallsAndRoof() {
		var w1    = wall(0,  0, 1, 12);
		var w2    = wall(15, 0, 1, 12);
		var roof  = wall(0, 0, 16, 1);
		w1.userData.isGround = false;
		w2.userData.isGround = false;
		roof.userData.isGround = false;
		return [w1, w2, roof];
	}
}